package player;
import flash.display.BlendMode;
import flixel.FlxObject;
import flixel.FlxSprite;
import flixel.group.FlxGroup;
import flixel.input.gamepad.FlxGamepad;
import flixel.util.FlxColor;
import flixel.util.FlxSpriteUtil;
import ice.entity.Component;
import flixel.FlxG;
import ice.entity.GameObject;
import ice.entity.GameObjectManager;
import flixel.input.gamepad.XboxButtonID;

/**
 * ...
 * @author 
 */
class PlayerLogic extends Component
{
	var moveSpeed:Int = 200;
	var jumpSpeed:Int = 300;
	var doubleJump:Bool = false;
	
	var enemies:FlxGroup;
	
	var lamp:FlxSprite;
	var dark:GameObject;
	
	var gamepad:FlxGamepad;
	
	var won:Bool = false;
	
	public function new(Owner : Int) 
	{
		super(Owner);
	}
	
	override private function init() 
	{
		super.init();
		owner.maxVelocity.x = 200;
		enemies = GameObjectManager.getInstance().GetGroup("enemies");
		GameObjectManager.map.setTileProperties(33, FlxObject.NONE, kill);
		GameObjectManager.map.setTileProperties(50, FlxObject.NONE, win);
	}
	
	override public function Update() 
	{
		super.Update();
		
		gamepad = FlxG.gamepads.lastActive;
		if (gamepad != null)
		{
			gamepad.deadZone = 0.3;
		}
		
		Collide();
		
		owner.velocity.y += Reg.gravity * FlxG.elapsed;
		
		if (owner.y > GameObjectManager.map.height - 1300)
		{
			kill();
		}
		
		HandleMovement();
	}
	
	function HandleMovement()
	{
		var moved:Bool = false;
		if (owner.alive && !won)
		{
			var useGampad = gamepad != null;

			if (FlxG.keys.justPressed.SPACE || useGampad && gamepad.justPressed(XboxButtonID.A))
			{
				if (owner.isTouching(FlxObject.FLOOR) || !doubleJump)
				{
					if (!doubleJump && !owner.isTouching(FlxObject.FLOOR))
					{
						owner.velocity.y = 0;
						doubleJump = true;
					}
					owner.velocity.y -= jumpSpeed;
				}
			}
			if (owner.isTouching(FlxObject.FLOOR))
			{
				doubleJump = false;
			}
			
			if (FlxG.keys.pressed.D || useGampad && gamepad.getAxis(XboxButtonID.LEFT_ANALOGUE_X) > 0)
			{
				if (owner.velocity.x < 0 && owner.isTouching(FlxObject.FLOOR))
				{
					owner.velocity.x = 0;
				}
				owner.facing = FlxObject.RIGHT;
				owner.velocity.x += moveSpeed * FlxG.elapsed;
				moved = true;
			}
			if (FlxG.keys.pressed.A || useGampad && gamepad.getAxis(XboxButtonID.LEFT_ANALOGUE_X) < 0)
			{
				if (owner.velocity.x > 0 && owner.isTouching(FlxObject.FLOOR))
				{
					owner.velocity.x = 0;
				}
				owner.facing = FlxObject.LEFT;
				owner.velocity.x -= moveSpeed * FlxG.elapsed;
				moved = true;
			}
		
			if(moved)
			{
				owner.drag.x = 0;
				
				if (owner.isTouching(FlxObject.FLOOR))
				{
					owner.animation.play("run");
				}
			}
			if (!moved && owner.isTouching(FlxObject.FLOOR))
			{
				owner.drag.x = owner.maxVelocity.x * 2;//HACKKKKKKKKKKK
				owner.animation.play("idle");
			}
			
			if (!owner.isTouching(FlxObject.FLOOR))
			{
				if (owner.velocity.y > 0)
				{
					owner.animation.play("fall");
				}
				else
				{
					owner.animation.play("jump");
				}
			}
		}
		else if(!won)
		{
			owner.drag.x = owner.maxVelocity.x * 2;//HACKKKKKKKKKKK
			if (owner.animation.finished)
			{
				PlayState.Reset();
			}
		}
		else
		{
			if (owner.animation.finished)
			{
				FlxG.switchState(new WinState());
			}
		}
	}
	
	function Collide():Void 
	{
		FlxG.collide(owner, GameObjectManager.map);
		for (e in enemies)
		{
			if (FlxG.overlap(owner, e))
			{
				kill(owner,cast e);//probably will kill performance
			}
		}
	}
	
	function kill(?obj:Null<FlxObject>, ?obj2:FlxObject):Void //Never used callbacks before, don't understand why objs are needed 
	{
		if (Std.is(obj2, Player) || Std.is(obj,Player))
		{
			die();
		}
		if (obj == null && obj2 == null)
		{
			die();
		}
	}
	
	function die()
	{
		if (owner.alive)
		{
			owner.animation.play("die");
			owner.alive = false;
		}
	}
	
	function win(?obj:Null<FlxObject>, ?obj2:FlxObject)
	{
		if (Std.is(obj2, Player) || Std.is(obj, Player))
		{
			if (owner.alive && !won)
			{
				owner.animation.play("win");
				won = true;
			}
		}
	}
}